# -*- coding: utf-8 -*-
import json
from django.http import HttpResponse, HttpResponseRedirect, Http404

import logging
import sys
log = logging.getLogger(__name__)

handler = logging.StreamHandler(stream=sys.stdout)
handler.setLevel(logging.DEBUG)

log.addHandler(handler)
log.setLevel(logging.DEBUG)

'''
Django에서 Response에 넣을 JSONObject 클래스
'''
class ArcReturnJSONObject:
    RESULT_STATE_SUCCESS = 'success';
    RESULT_STATE_FAILURE = 'failure';

    dict = {
        'result': RESULT_STATE_SUCCESS,
        'message': '',
        'data': {},
    }

    '''
    생성자
        인자가 없을 경우 : 성공여부 success로 시작
        인자가 1개 : 성공여부 추가
        인자가 2개 : 성공여부, 메시지 추가
    '''
    def __init__(self, *args):
        self.dict.clear()
        self.dict['result'] = self.RESULT_STATE_FAILURE
        self.dict['message'] = ''
        self.dict['data'] = {}

        len_args = len(args)
        if len_args == 0:
            pass
        if len_args >= 1:
            if isinstance(args[0], bool):
                if args[0]:
                    self.dict['result'] = self.RESULT_STATE_SUCCESS
                elif not args[0]:
                    self.dict['result'] = self.RESULT_STATE_FAILURE
        if len_args == 2:
            if isinstance(args[1], str) or isinstance(args[1], unicode):
                self.dict['message'] = args[1]
        

    # 결과가 성공인지, 실패인지
    def set_state(self, value):
        if value:
            self.dict['result'] = self.RESULT_STATE_SUCCESS
        else:
            self.dict['result'] = self.RESULT_STATE_FAILURE

    def set_message(self, value):
        self.dict['message'] = value

    # Object 삽입
    def add_object(self, key, value):
        self.dict['data'][key] = value

    # .json()함수가 있는 경우, 해당 value를 .json()후 넣어줌
    def add_object_to_json(self, key, value):
        self.dict['data'][key] = value.json()

    def add_array_to_json(self, key, array):
        data_array = []
        for item in array:
            data_array.append(item.json())
        self.dict['data'][key] = data_array

    def json_response(self):
        return HttpResponse(json.dumps(self.dict), content_type='application/json')

    def json_dumps(self):
        return json.dumps(self.dict)

    def json_loads(self, string):
        dict = json.loads(string)
        try:
            result = dict['result']
            message = dict['message']
            data = dict['data']

            self.dict['result'] = result
            self.dict['message'] = message
            self.dict['data'] = data
        except Exception as e:
            log.debug(e.args)

    def get_result(self):
        return self.dict['result']

    def get_message(self):
        return self.dict['message']

    def get_data(self):
        return self.dict['data']





# class ArcImage:
    


# -*- coding: utf-8 -*-
import json

from datetime import datetime, time, date

from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect, Http404

# datetime -> String 변환 함수 ###
def day_to_string(value):
    if isinstance(value, datetime) or isinstance(value, date):
        return value.strftime('%Y.%m.%d')
    else:
        return u'None'

def minute_to_string(value):
    if isinstance(value, datetime):
        return value.strftime('%Y.%m.%d %H:%M')
    else:
        return u'None'

def time_to_string(value):
    if isinstance(value, datetime):
        return value.strftime('%H:%M')
    else:
        return u'None'

def datetime_to_string(instance, string_format='%Y.%m.%d %H:%M:%S'):
    if isinstance(instance, datetime):
        return instance.strftime(string_format)
    else:
        return '0000.00.00 00:00:00'



def imageinfo(instance, url=None):
    dict = {
        'has_image': True if instance else False,
        'width': instance.width if instance else '',
        'height': instance.height if instance else '',
        'url': instance.url if instance else url,
    }
    if not instance and url is None:
        dict['url'] = ''
    return dict


'''
    두 점 사이의 거리 계산 공식
    Calculate the great circle distance between two points 
    on the earth (specified in decimal degrees)
'''
from math import radians, cos, sin, asin, sqrt
def distance(lat1, lon1, lat2, lon2):
    # convert decimal degrees to radians 
    lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

    # haversine formula 
    dlon = lon2 - lon1 
    dlat = lat2 - lat1 
    a = sin(dlat/2)**2 + cos(lat1) * cos(lat2) * sin(dlon/2)**2
    c = 2 * asin(sqrt(a)) 

    # 6367 km is the radius of the Earth
    km = 6367 * c
    return km 

'''
페이스북 사진 가져오는 함수
'''
def get_facebook_profile_url(facebook_id):
    url = 'https://graph.facebook.com/%s/picture?type=large' % (facebook_id)
    return url
    
def get_facebook_profile_image(facebook_id):
    url = 'https://graph.facebook.com/%s/picture?type=large' % (facebook_id)




#-*- coding: utf-8 -*-
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods

from arclib.django.object import ArcReturnJSONObject

def api(func):
    @csrf_exempt
    # @require_http_methods(["POST"])
    def wrap(request, *args, **kwargs):
        return func(request, *args, **kwargs)

    wrap.__doc__ = func.__doc__
    wrap.__name__ = func.__name__
    return wrap

def exception_json(func):
    def wrap(request, *args, **kwargs):
        try:
            print 'exception_json try : %s' % (func.__name__)
            return func(request, *args, **kwargs)
        except Exception as e:
            print 'exception_json except : %s, %s' % (func.__name__, str(e.__class__))
            try:    exception_class = str(e.__class__)
            except: exception_class = 'get class failure'

            try:    exception_type = str(type(e))
            except: exception_type = 'get type failure'

            try:    exception_args = str(e.args)
            except: exception_args = 'get args failure'

            try:    exception = str(e)
            except: exception = 'get error failure'

            try:    str_request = str(request)
            except: str_request = 'get request failure'

            try:    str_request_get = str(request.GET)
            except: str_request_get = 'get request.GET failure'

            try:    str_request_post = str(request.POST)
            except: str_request_post = 'get request.POST failure'

            try:    str_request_files = str(request.FILES)
            except: str_request_files = 'get request.FILES failure'

            try:    str_request_path = str(request.path)
            except: str_request_path = 'get request.path failure'

            dict = {
                'class': exception_class,
                'type': exception_type,
                'exception': exception,
                'args': exception_args,
                'request': str_request,
                'request_get': str_request_get,
                'request_post': str_request_post,
                'request_files': str_request_files,
                'request_path': str_request_path,
            }
            jsondata = ArcReturnJSONObject(False, 'exception_json')
            jsondata.add_object('exception', dict)
            return jsondata.json_response()
    return wrap


# -*- coding: utf-8 -*-
from django.utils.six import with_metaclass
from django.db import models
from timedelta.fields import TimedeltaField

def imageinfo(instance, url=None):
    dict = {
        'has_image': True if instance else False,
        'width': instance.width if instance else '',
        'height': instance.height if instance else '',
        'url': instance.url if instance else url,
    }
    if not instance and url is None:
        dict['url'] = ''
    return dict

class ArcImageField(with_metaclass(models.SubfieldBase, models.ImageField)):
    def __init__(self, *args, **kwargs):
        super(ArcImageField, self).__init__(*args, **kwargs)

    def imageinfo(self, url=None):
        return imageinfo(self, url)

class ArcTimedeltaField(TimedeltaField):
    def __init__(self, *args, **kwargs):
        super(ArcTimedeltaField, self).__init__(*args, **kwargs)

    def years(self):
        seconds = self.total_seconds()
        years = seconds / (3600 * 24 * 365)
        return years

    def days(self):
        seconds = self.total_seconds()
        years = self.years()
        remain_seconds = seconds - (years * 3600 * 24 * 365)
        days = remain_seconds / (3600 * 24)
        return days

    def hours(self):
        seconds = self.total_seconds()
        years = self.years()
        days = self.days()
        remain_seconds = seconds - (years * 3600 * 24 * 365) - (days * 3600 * 24)
        hours = remain_seconds / 3600
        return hours

    def minutes(self):
        seconds = self.total_seconds()
        seconds = self.total_seconds()
        years = self.years()
        days = self.days()
        hours = self.hours()
        remain_seconds = seconds - (years * 3600 * 24 * 365) - (days * 3600 * 24) - (hours * 3600)
        minutes = remain_seconds / 60
        return minutes


# -*- coding: utf-8 -*-
import json
from google.appengine.api import memcache
CACHE_SIZE = 900000
FORMAT_KEY_NAME = '%s_split_%d'
FORMAT_KEY_COUNT = '%s_split_count'
FORMAT_KEY_TYPE = '%s_type'
TYPE_JSON = 'json'
TYPE_STRING = 'string'

'''
memcache 커스터마이징
자동으로 최대 캐시 크기만큼 split해서 저장/리턴 해준다
'''

def set_large(key, value, namespace=None):
    print '-- memcache.set start --'
    print '\tkey : %s' % (key)
    count = 0
    print '\tvalue : %s' % (value)
    print '\tdata length : %s' % (len(value))

    if isinstance(value, dict) or isinstance(value, list):
        data = json.dumps(value)
        memcache.set(namespace=namespace, key=FORMAT_KEY_TYPE % (key), value=TYPE_JSON)
    elif isinstance(value, str) or isinstance(value, unicode):
        data = value
        memcache.set(namespace=namespace, key=FORMAT_KEY_TYPE % (key), value=TYPE_JSON)
    else:
        return False

    while len(data) > 0:
        print '\t\twhile loop %s' % (count)
        cur_data = data[0:CACHE_SIZE]
        data = data[CACHE_SIZE:]
        cur_key_name = FORMAT_KEY_NAME % (key, count)
        memcache.set(namespace=namespace, key=cur_key_name, value=cur_data)
        count += 1
    cur_key_count = FORMAT_KEY_COUNT % (key)
    print '\tall count : %s' % (count)
    memcache.set(namespace=namespace, key=cur_key_count, value=count)
    return True
    # try:
    #     data = value
    #     count = 0
    #     while len(value) > 0:
    #         cur_data = data[0:CACHE_SIZE]
    #         data = data[CACHE_SIZE:]
    #         cur_key_name = FORMAT_KEY_NAME % (key, count)
    #         memcache.set(namespace=namespace, key=cur_key_name, value=cur_data)
    #         count += 1
    #     cur_key_count = FORMAT_KEY_COUNT % (key)
    #     memcache.set(namespace=namespace, key=cur_key_count, value=count)
    #     return True
    # except Exception as e:
    #     print type(e)
    #     print e.args
    #     print e
    #     return False

def get_large(key, namespace=None, convert_dict=None):
    print '-- memcache.get start --'
    cur_key_count = FORMAT_KEY_COUNT % (key)
    count = memcache.get(namespace=namespace, key=cur_key_count)
    type = memcache.get(namespace=namespace, key=FORMAT_KEY_TYPE % (key))
    print '\tkey : %s' % (key)

    if count is None:
        print '\tcount is None, return None'
        return None

    print '\tcount : %s' % (count)

    data_combine = ''
    for i in range(count):
        print '\t\tfor loop %s/%s' % (i + 1, count)
        cur_key_name = FORMAT_KEY_NAME % (key, i)
        cur_data = memcache.get(namespace=namespace, key=cur_key_name)

        if cur_data is None:
            print '\tfor loop except, cur_data is None. return None'
            return None
        data_combine += cur_data

    if type == TYPE_JSON and convert_dict == True:
        return json.loads(data_combine)
    else:
        return data_combine